// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATLIST__IPP
#define BOOST__IO__FORMATLIST__IPP
   // basic format manipulator:

   template< typename ForwardIterator >
   boost::io::formatlist_t< ForwardIterator > boost::io::formatlist
   (
      ForwardIterator first,
      ForwardIterator last
   )
   {
      return( formatlist_t< ForwardIterator >( first, last ));
   }

   template< class Container >
   boost::io::formatlist_t< typename Container::iterator > boost::io::formatlist
   (
      Container & c
   )
   {
      return( formatlist_t< typename Container::iterator >( c.begin(), c.end()));
   }

   // advanced format manipulator:

   template< typename ForwardIterator, typename FormatType, class RefType >
   boost::io::formatlist_t< ForwardIterator, FormatType > boost::io::formatlist
   (
      ForwardIterator first,
      ForwardIterator last,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< ForwardIterator, FormatType >( first, last, fmt ));
   }

   template< class Container, typename FormatType, class RefType >
   boost::io::formatlist_t< typename Container::iterator, FormatType > boost::io::formatlist
   (
      Container & c,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< typename Container::iterator, FormatType >( c.begin(), c.end(), fmt ));
   }

   // extended format manipulator:

   template< typename FormatType, typename ForwardIterator >
   boost::io::formatlist_t< ForwardIterator, FormatType > boost::io::formatlistex
   (
      ForwardIterator first,
      ForwardIterator last
   )
   {
      return( formatlist_t< ForwardIterator, FormatType >( first, last ));
   }

   template< typename FormatType, class Container >
   boost::io::formatlist_t< typename Container::iterator, FormatType > boost::io::formatlistex
   (
      Container & c
   )
   {
      return( formatlist_t< typename Container::iterator, FormatType >( c.begin(), c.end()));
   }

   // format manipulator -- output functor versions:    

   template< typename ForwardIterator, class Outputter >
   boost::io::formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
   boost::io::formatlistfn
   (
      ForwardIterator first,
      ForwardIterator last,
      Outputter       out
   )
   {
      return( formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
      (
         first,
         last,
         out
      ));
   }

   template< class Container, class Outputter >
   boost::io::formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
   boost::io::formatlistfn
   (
      Container & c,
      Outputter   out
   )
   {
      return( formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
      (
         c.begin(),
         c.end(),
         out
      ));
   }
#endif
