// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMAT_OBJECTS__LIST_OUTPUT__HPP
#define BOOST__IO__FORMAT_OBJECTS__LIST_OUTPUT__HPP
   namespace boost { namespace io { namespace detail
   {
      template< typename FormatType, class RetType, typename Outputter = boost::io::basic_output >
      class list_output: public formatter_t< FormatType, RetType >
      {
         public:
            typedef list_output< FormatType, RetType, Outputter >    this_type;
            typedef typename formatter_t< FormatType, this_type >::traits_type
                                                                     traits_type;
         public:
            Outputter                  outputter;
         public:
            template< typename ForwardIterator, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream &  os,
                                     ForwardIterator first,
                                     ForwardIterator last
                                  ) const
            {
               if( traits_type::isok( openlist()))
                  os << openlist();
               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               while( first != last )
               {
                  outputter( os, *first );

                  if( ++first != last )
                  {
                     if( space_before_sep())
                        os << static_cast< typename OutputStream::char_type >( ' ' );
                     if( traits_type::isok( separator()))
                        os << separator();
                     if( space_after_sep())
                        os << static_cast< typename OutputStream::char_type >( ' ' );
                  }
               }

               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );
               if( traits_type::isok( closelist()))
                  os << closelist();
               return( os );
            }
         public:
            inline           list_output() throw()
            {
            }
            inline           list_output( Outputter o ) throw():
               outputter( o )
            {
            }
            inline           list_output( const RetType & o ) throw():
               formatter_t< FormatType, RetType >( o ),
               outputter( o.outputter )
            {
            }
            template< typename RT2 >
            inline           list_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               formatter_t< FormatType, RetType >( fmt )
            {
            }
            template< typename RT2 >
            inline           list_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt,
                                Outputter o
                             ) throw():
               formatter_t< FormatType, RetType >( fmt ),
               outputter( o )
            {
            }
      };
   }}}
#endif
