// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMAT_OBJECTS__STDPAIR_OUTPUT__HPP
#define BOOST__IO__FORMAT_OBJECTS__STDPAIR_OUTPUT__HPP
#  include <utility>
#  include <complex>

   namespace boost { namespace io
   {
      template
      <
         typename FormatType,
         typename Outputter1 = boost::io::basic_output,
         typename Outputter2 = boost::io::basic_output
      >
      class stdpair_output: public formatter_t< FormatType, stdpair_output
                                                            <
                                                               FormatType,
                                                               Outputter1,
                                                               Outputter2
                                                            > >
      {
         public:
            typedef stdpair_output< FormatType, Outputter1, Outputter2 >
                                                                     this_type;
            typedef typename formatter_t< FormatType, this_type >::traits_type
                                                                     traits_type;
         public:
            Outputter1                 firstout;
            Outputter2                 secondout;
         public:
            template< typename T1, typename T2, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const std::pair< T1, T2 > & p
                                  ) const
            {
               return( out( os, p.first, p.second ));
            }
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const std::complex< T > & p
                                  ) const
            {
               return( out( os, p.real(), p.imag()));
            }
         private:
            template< typename T1, typename T2, class OutputStream >
            inline OutputStream &                out
                                                 (
                                                    OutputStream & os,
                                                    const T1 &     first,
                                                    const T2 &     second
                                                 ) const
            {
               if( traits_type::isok( openlist()))
                  os << openlist();
               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               firstout( os, first );

               if( space_before_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );
               if( traits_type::isok( separator()))
                  os << separator();
               if( space_after_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               secondout( os, second );

               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );
               if( traits_type::isok( closelist()))
                  os << closelist();

               return( os );
            }
         public:
            inline                     stdpair_output() throw()
            {
            }
            inline                     stdpair_output( Outputter1 o1 ) throw():
               firstout( o1 )
            {
            }
            inline                     stdpair_output( Outputter1 o1, Outputter2 o2 ) throw():
               firstout(  o1 ),
               secondout( o2 )
            {
            }
            inline                     stdpair_output( const stdpair_output & o ) throw():
               formatter_t< FormatType, stdpair_output >( o ),
               firstout(  o.firstout  ),
               secondout( o.secondout )
            {
            }
            template< typename RT2 >
            inline           stdpair_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               formatter_t< FormatType, stdpair_output >( fmt )
            {
            }
      };
   }}
#endif
