// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATLIST__HPP
#define BOOST__IO__FORMATLIST__HPP
#  include <boost/config.hpp>

#  include "detail/format_traits.hpp"
#  include "formatter.hpp"
#  include "format_objects.hpp"
#  include "formatlist_t.hpp"

   namespace boost { namespace io
   {
      // basic format manipulator:
      // NOTE: If you want a wchar_t formatter, you must use formatlistex or a formatter object.

      template< typename ForwardIterator >
      inline formatlist_t< ForwardIterator >     formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< class Container >
      inline formatlist_t< typename Container::iterator >
                                                 formatlist
                                                 (
                                                    Container &
                                                 );

      // advanced format manipulator:

      template< typename ForwardIterator, typename FormatType, class RefType >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    const formatter_t< FormatType, RefType > & 
                                                 );

      template< class Container, typename FormatType, class RefType >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlist
                                                 (
                                                    Container &,
                                                    const formatter_t< FormatType, RefType > &
                                                 );

      // extended format manipulator:

      template< typename FormatType, typename ForwardIterator >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlistex
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< typename FormatType, class Container >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlistex
                                                 (
                                                    Container & 
                                                 );

      // format manipulator -- output functor versions:

      template< typename ForwardIterator, class Outputter >
      inline formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    Outputter
                                                 );
      
      template< class Container, class Outputter >
      inline formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    Container &,
                                                    Outputter
                                                 );
   }}

#  include "detail/formatlist.ipp"
#endif
