// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATLIST_T__HPP
#define BOOST__IO__FORMATLIST_T__HPP
#  include <iostream>
#  include <iterator>

   namespace boost { namespace io
   {
      template
      <
         typename ForwardIterator, typename FormatType = char,
         typename Outputter = boost::io::basic_output
      >
      class formatlist_t: public detail::list_output< FormatType, formatlist_t
                                                                  <
                                                                     ForwardIterator,
                                                                     FormatType,
                                                                     Outputter
                                                                  >, Outputter >
      {
         private:
            ForwardIterator            first;
            ForwardIterator            last;
         public:
            inline ForwardIterator               begin() const throw();
            inline ForwardIterator               end()   const throw();
         public:
            typedef ForwardIterator                        iterator;
         public: // constructors
            inline           formatlist_t( ForwardIterator f, ForwardIterator l ) throw();
            inline           formatlist_t( ForwardIterator f, ForwardIterator l, Outputter ) throw();
            template< typename RT2 >
            inline           formatlist_t
                             (
                                ForwardIterator f,
                                ForwardIterator l,
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               detail::list_output< FormatType, formatlist_t, Outputter >( fmt ),
               first( f ),
               last(  l )
            {
            }
            template< typename RT2 >
            inline           formatlist_t
                             (
                                ForwardIterator f,
                                ForwardIterator l,
                                const formatter_t< FormatType, RT2 > & fmt,
                                Outputter out
                             ) throw():
               detail::list_output< FormatType, formatlist_t, Outputter >( fmt, out ),
               first( f ),
               last(  l )
            {
            }
      };

      template< typename CharT, class TraitsT, typename ForwardIterator, typename FormatType, typename Outputter >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT > &,
         const formatlist_t< ForwardIterator, FormatType, Outputter > &
      );
   }}

#  include "detail/formatlist_t.ipp"
#endif
