// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATTER__HPP
#define BOOST__IO__FORMATTER__HPP
   namespace boost { namespace io
   {
      template< typename FormatType, class ReferenceType >
      class formatter_t
      {
         public:
            typedef FormatType                             listfmt_type;
            typedef detail::format_traits< listfmt_type >  traits_type;
         private: // format control
            listfmt_type               fmt_openlist;
            listfmt_type               fmt_closelist;
            listfmt_type               fmt_separator;
         public:
            inline listfmt_type                  openlist()  const throw();
            inline listfmt_type                  closelist() const throw();
            inline listfmt_type                  separator() const throw();
         public:
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type, // close list
                                                    listfmt_type  // seperator
                                                 ) throw();
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type  // close list
                                                 ) throw();
            inline ReferenceType &               format
                                                 (
                                                    listfmt_type  // seperator
                                                 ) throw();
         private: // spacing control
            bool                       fmt_spaceBeforeSep;
            bool                       fmt_spaceAfterSep;
            bool                       fmt_spaceAroundList;
         public:
            inline bool                          space_before_sep()  const throw();
            inline bool                          space_after_sep()   const throw();
            inline bool                          space_around_list() const throw();
         public:
            inline ReferenceType &               space
                                                 (
                                                    bool, // before separator
                                                    bool, // after  separator
                                                    bool  // around list
                                                 ) throw();
            inline ReferenceType &               space
                                                 (
                                                    bool, // before separator
                                                    bool  // after  separator
                                                 ) throw();
            inline ReferenceType &               space
                                                 (
                                                    bool  // around list
                                                 ) throw();
         public: // configure from a formatter object
            template< typename RT2 >
            inline ReferenceType &               format
                                                 (
                                                    const formatter_t< FormatType, RT2 > & fmt
                                                 )
            {
               return( format( fmt.openlist(), fmt.closelist(), fmt.separator()));
            }
            template< typename RT2 >
            inline ReferenceType &               space
                                                 (
                                                    const formatter_t< FormatType, RT2 > & fmt
                                                 )
            {
               return( space( fmt.space_before_sep(), fmt.space_after_sep(), fmt.space_around_list()));
            }
            template< typename RT2 >
            inline ReferenceType &               inherit
                                                 (
                                                    const formatter_t< FormatType, RT2 > & fmt
                                                 )
            {
               format( fmt.openlist(), fmt.closelist(), fmt.separator());
               return( space( fmt.space_before_sep(), fmt.space_after_sep(), fmt.space_around_list()));
            }
         public: // constructors
            inline           formatter_t() throw();
            template< typename RT2 >
            inline           formatter_t( const formatter_t< FormatType, RT2 > & fmt ) throw():
               fmt_openlist(        fmt.openlist()),
               fmt_closelist(       fmt.closelist()),
               fmt_separator(       fmt.separator()),
               fmt_spaceBeforeSep(  fmt.space_before_sep()),
               fmt_spaceAfterSep(   fmt.space_after_sep()),
               fmt_spaceAroundList( fmt.space_around_list())
            {
            }
      };

      template< typename FormatType >
      class formatter: public formatter_t< FormatType, formatter< FormatType > >
      {
         public:
            inline           formatter() throw();
            template< typename RT2 >
            inline           formatter( const formatter_t< FormatType, RT2 > & fmt ) throw():
               formatter_t< FormatType, formatter >( fmt )
            {
            }
      };
   }}

#  include "detail/formatter.ipp"
#endif
