// (C) Copyright 2003: Reece H. Dunn 

// Test file for boost/io/formatlist.hpp

#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

int main()
{
   // std::vector

   std::vector< int >                  v( 10 );
   std::generate( v.begin(), v.end(), std::rand );

   // std::list

   std::list< char >                   l;
   l.push_back( 'A' );
   l.push_back( 'B' );
   l.push_back( 'C' );

   // std::string and arrays

   const char                        * str  = "Jeanette Biedermann";
   std::string                         s    = "Warum bist du nicht da?";

   int                                 i[ 4 ] = { 3, 6, 9, 12 };

   // examples

   std::cout << "vector< int > = " << boost::io::formatlist( v ).format( '(', ')' ) << '\n';
   std::cout << "vector< int > = " << boost::io::formatlist( v ).format( '|' ).space( true, true ) << '\n';

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( i, i + 4 ).format( ';' ).space( true, true )
             << '\n';

   std::cout << "char * = "
             << boost::io::formatlist( str, str + std::strlen( str )).format( ';' ).space( false )
             << '\n';

   std::cout << "std::string = "
             << boost::io::formatlist( s ).format( '\0', '\0', '|' ).space( false, false, false )
             << '\n';

   std::cout << '\n' << "sub-list:" << '\n' << '\n';

   std::cout << "std::string = "
             << boost::io::formatlist( s.begin() + 6, s.end()).space( false, false, false ).format( '"', '"', '-' )
             << '\n';

#  if (FORMATLIST__VERSION >= 0x0104) // new examples 04/04/2003:
      std::cout << '\n' << "advanced formatting:" << '\n' << '\n';

      std::cout << "std::string = "
                << boost::io::formatlistex< const char * >( s ).format( "<:", ":>", "\0" ) << '\n';

      std::cout << "std::string = "
                << boost::io::formatlistex< const char * >( s )
                   .format( "{ '", "' }", "', '" )
                   .space( false, false, false )
                << '\n';

      std::cout << "int[ 4 ] = "
                << boost::io::formatlistex< const char * >( i, i + 4 ).format( "-:-" ).space( true, true ) << '\n';

      std::cout << "int[ 4 ] = "
                << boost::io::formatlistex< const char * >( i, i + 4 )
                   .format( "-\n-" ).format( "\n-", "-" )
                   .space( true, true )
                << '\n';
#  endif

   return( 0 );
}
