// (C) Copyright 2003: Reece H. Dunn 

// Test file for boost/io/formatlist.hpp

#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

int main()
{
   // std::vector

   std::vector< int >                  v( 10 );
   std::generate( v.begin(), v.end(), std::rand );

   // std::list

   std::list< char >                   l;
   l.push_back( 'A' );
   l.push_back( 'B' );
   l.push_back( 'C' );

   // std::string and arrays

   const char                        * str  = "Jeanette Biedermann";
   std::string                         s    = "Warum bist du nicht da?";

   int                                 i[ 4 ] = { 3, 6, 9, 12 };

   // formatter configurations

   boost::io::formatter< char >        fmt;
   fmt.format( '(', ')' );
   fmt.space(  true, true );

   boost::io::formatter< char * >      fmt2;
   fmt2.format( "--" );
   fmt2.space(  true, true );

   // examples

   // Mechanism #1: pass a formatter object to the formatlist object:

   std::cout << "mechanism 1:" << '\n';

   std::cout << "int[ 4 ] = " << boost::io::formatlist( i, i + 4 ).format( fmt ) << '\n';
   std::cout << "char * = "   << boost::io::formatlist( str, str + std::strlen( str )).format( fmt ) << '\n';

   std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).format(  fmt ) << '\n'; // format  only
   std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).space(   fmt ) << '\n'; // spacing only
   std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).inherit( fmt ) << '\n'; // format *and* spacing

   // Mechanism #2: pass a formatter object to the formatlist manipulator:

   std::cout << '\n' << "mechanism 1 and 2 side-by-side:" << '\n';

   // NOTE: These two are equivalent, but the second will be more efficient
   // and has less typing

   std::cout << "vector< int > = " << boost::io::formatlist( v ).inherit( fmt ) << '\n';
   std::cout << "vector< int > = " << boost::io::formatlist( v, fmt ) << '\n';

   // These two are also equivalent:
   // NOTE: The second version is shorter since it gets the format type from
   // the formatter object

   std::cout << "int[ 4 ] = " << boost::io::formatlistex< char * >( i, i + 4 ).inherit( fmt2 ) << '\n';
   std::cout << "int[ 4 ] = " << boost::io::formatlist( i, i + 4, fmt2 ) << '\n';

   return( 0 );
}
