// (C) Copyright 2003: Reece H. Dunn 

// Test file for boost/io/formatlist.hpp

#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

int main()
{
   // std::map

   std::map< std::string, std::string >
                                       m;
   m[ "Abscheid Nehmen"    ] = "Farewell";
   m[ "Engel"              ] = "Angel";
   m[ "Mas Alla"           ] = "Beyond";
   m[ "Wenn Das Liebe Ist" ] = "If This Is Love";

   // std::list< std::complex >

   std::list< std::complex< double > > dl;
   dl.push_front( std::complex< double >( 1.0, 2.5 ));
   dl.push_front( std::complex< double >( 7.9, 0.0 ));
   dl.push_front( std::complex< double >( 3.5, 9.7 ));

   // formatter configurations

   boost::io::stdpair_output< char >   pairfmt1;
   pairfmt1.format( '(', ')' );

   boost::io::stdpair_output< char * > pairfmt2;
   pairfmt2.format( "(", ")", "|" ).space( true, true );

   boost::io::stdpair_output< char * > pairfmt3;
   pairfmt3.format( "\0", "\0", "==>" ).space( true, true, false );

   // examples

   std::cout << "map< string, string > = " << boost::io::formatlistfn( m, pairfmt1 ) << '\n';

   std::cout << "map< string, string > = "
             << boost::io::formatlistfn( m, pairfmt1 )
                .format( '\n', '\0', '\n' ).space( false, false, false )
             << '\n' << '\n';

   std::cout << "map< string, string > = "
             << boost::io::formatlistfn( m, pairfmt2 )
                .format( "\n==>", "\0", "\n==>" ).space( false, true, true )
             << '\n' << '\n';

   std::cout << "map< string, string > = "
             << boost::io::formatlistfn( m, pairfmt3 )
                .format( "\n   ", "\0", "\n   " ).space( false, false, false )
             << '\n' << '\n';

   std::cout << "list< complex< double > > = " << boost::io::formatlist(   dl ) << '\n';
   std::cout << "list< complex< double > > = " << boost::io::formatlistfn( dl, pairfmt1 ) << '\n';
   std::cout << "list< complex< double > > = " << boost::io::formatlistfn( dl, pairfmt2 ) << '\n';

   return( 0 );
}
